part of '../machine.dart';

typedef BuilderTransitionCondition<TValue, TEvent, TContext, TE extends TEvent>
    = bool Function(TContext context, TE event);

typedef BuilderTransitionReduce<TValue, TEvent, TContext, TE extends TEvent>
    = TContext Function(TContext context, TE event);

class MachineBuilder<TValue, TEvent, TContext>
    extends BuilderBase<TValue, TEvent, TContext> {
  TransitionEvent<TValue, TEvent, TContext> onEvent<TE extends TEvent>(
    TValue to, {
    BuilderTransitionReduce<TValue, TEvent, TContext, TE>? reduce,
    BuilderTransitionCondition<TValue, TEvent, TContext, TE>? condition,
  }) {
    return TransitionEvent<TValue, TEvent, TContext>(
      to: to,
      isCandidate: (EventType<TValue, TEvent, TContext> obj) =>
          obj.maybeMap(custom: (c) => c.event is TE, orElse: () => false),
      reduce: null != reduce
          ? (ctx, e) {
              final _event = e as EventTypeCustom<TValue, TEvent, TContext>;
              final _tEvent = _event.event as TE;
              return reduce(ctx, _tEvent);
            }
          : null,
      condition: null != condition
          ? (ctx, e) {
              final _event = e as EventTypeCustom<TValue, TEvent, TContext>;
              final _tEvent = _event.event as TE;
              return condition(ctx, _tEvent);
            }
          : null,
    );
  }
}
