part of 'machine.dart';

@freezed
class Log with _$Log {
  factory Log.machineStartInitially(String machineType, MachineState state) =
      _StartMachineInitially;
  factory Log.machineStartStopped(String machineType, MachineState state) =
      _StartMachineStopped;
  factory Log.machineStartManaged(
          String machineType, String parentMachineType, MachineState state) =
      _StartMachineManaged;
  factory Log.machineStop(String machineType, MachineState state) =
      _StopMachine;
  factory Log.machineStopManaged(
          String machineType, String parentMachineType, MachineState state) =
      _StopMachineManaged;
  factory Log.transition(
      String machineType, MachineState from, MachineState to) = _Transition;
  factory Log.sendEvent(String machineType, dynamic event) = _SendEvent;
  factory Log.executeEffects(String machineType, MachineState state) =
      _ExecuteEffects;
  factory Log.effectSwallowEvent(String machineType, dynamic event) =
      _EffectSwallowEvent;
}

abstract class DebugLogger {
  void log(Log log);
}

class PrintLogger extends DebugLogger {
  @override
  void log(Log log) => log.map(
        machineStartInitially: (obj) => print(
            'Initially starting "${obj.machineType}" in "${obj.state.value}"'),
        machineStartStopped: (obj) => print(
            'Starting previously stopped "${obj.machineType}" in "${obj.state.value}"'),
        machineStartManaged: (obj) => print(
            'Starting managed "${obj.machineType}" in "${obj.state.value}" in parent "${obj.parentMachineType}"'),
        machineStop: (obj) =>
            print('Stop "${obj.machineType}" in "${obj.state.value}"'),
        machineStopManaged: (obj) => print(
            'Stop managed "${obj.machineType}" in "${obj.state.value}" in parent "${obj.parentMachineType}"'),
        transition: (obj) => print(
            'Transition from "${obj.from.value}" to "${obj.to.value}" in "${obj.machineType}"'),
        sendEvent: (obj) =>
            print('Sending event "${obj.event}" to "${obj.machineType}"'),
        executeEffects: (obj) => print(
            'Executing effects in "${obj.machineType}" in state "${obj.state.value}"'),
        effectSwallowEvent: (obj) => print(
            'Effect swallowed event "${obj.event}" meant for "${obj.machineType}"'),
      );
}
