part of 'machine.dart';

typedef EffectEnter<TValue, TEvent, TContext>
    = EffectLeave<TValue, TEvent, TContext>? Function(
        EffectValue<TValue, TEvent, TContext> effect);
typedef EffectLeave<TValue, TEvent, TContext> = void Function(
    MachineState<TValue, TEvent, TContext> state);

class _MachineEffect<TValue, TEvent, TContext> {
  _MachineEffect(this._enter);

  final EffectEnter<TValue, TEvent, TContext> _enter;
  EffectLeave<TValue, TEvent, TContext>? _leave;
  EffectValue<TValue, TEvent, TContext>? _curValue;

  void execute(MachineBase<TValue, TEvent, TContext> machine,
      MachineState<TValue, TEvent, TContext> stateWhenExecuted) {
    assert(null == _curValue,
        '${_typeOf<_MachineEffect<TValue, TEvent, TContext>>()} is already running. Something went very wrong ');

    _curValue = EffectValue<TValue, TEvent, TContext>(machine);
    _leave = _enter(_curValue!);
  }

  void kill(MachineState<TValue, TEvent, TContext> state) {
    _leave?.call(state);
    _curValue?.dispose();
    _curValue = null;
  }
}

class EffectValue<TValue, TEvent, TContext> {
  EffectValue(this._machine);

  final MachineBase<TValue, TEvent, TContext> _machine;
  bool _disposed = false;

  void dispose() {
    _disposed = true;
  }

  void _send(TEvent event) {
    assert(() {
      if (!isActive || !_machine.canTransition(event)) {
        _machine._log(Log.effectSwallowEvent(_machine._machineType, event));
      }
      return true;
    }(), '');
    if (isActive && _machine.canTransition(event)) {
      _machine.send(event);
    }
  }

  void Function(TEvent event) get send => _send;

  bool get isActive => !_disposed;

  MachineState<TValue, TEvent, TContext> getState() => _machine.status.maybeMap(
        orElse: () => throw Exception(
            '${_typeOf<EffectValue<TValue, TEvent, TContext>>()} is trying to access state in an invalid machine state ${_machine.status}'),
        running: (running) => running.state,
      );
}
