part of 'machine.dart';

typedef ReduceOnEnter<TValue, TEvent, TContext> = TContext Function(TContext);
typedef ReduceOnExit<TValue, TEvent, TContext> = TContext Function(TContext);
typedef TransitionReduce<TValue, TEvent, TContext> = TContext Function(
    TContext context, EventType<TValue, TEvent, TContext> event);
typedef TransitionImmediateReduce<TValue, TEvent, TContext> = TContext Function(
    TContext context);
typedef TransitionCondition<TValue, TEvent, TContext> = bool Function(
    TContext context, EventType<TValue, TEvent, TContext> event);
typedef TransitionImmediateCondition<TValue, TEvent, TContext> = bool Function(
    TContext context);

class StateNode<TValue, TEvent, TContext> {
  StateNode(this.config);

  StateNodeConfig<TValue, TEvent, TContext> config;

  TransitionImmediate<TValue, TEvent, TContext>? getImmediateCandidate(
          TContext context) =>
      config.immediate
          .firstWhereOrNull((t) => t.condition?.call(context) ?? true);

  TransitionEvent<TValue, TEvent, TContext>? getEventCandidate(
          EventType<TValue, TEvent, TContext> eventType, TContext context) =>
      config.transition
          .where((element) => element.isCandidate(eventType))
          .firstWhereOrNull(
              (t) => t.condition?.call(context, eventType) ?? true);
}

@freezed
class Transition<TValue, TEvent, TContext>
    with _$Transition<TValue, TEvent, TContext> {
  factory Transition.immediate({
    required TValue to,
    TransitionImmediateReduce<TValue, TEvent, TContext>? reduce,
    TransitionImmediateCondition<TValue, TEvent, TContext>? condition,
  }) = TransitionImmediate<TValue, TEvent, TContext>;

  factory Transition.event({
    required TValue to,
    required bool Function(EventType<TValue, TEvent, TContext> eventType)
        isCandidate,
    TransitionReduce<TValue, TEvent, TContext>? reduce,
    TransitionCondition<TValue, TEvent, TContext>? condition,
  }) = TransitionEvent<TValue, TEvent, TContext>;
}

@freezed
class StateNodeConfig<TValue, TEvent, TContext>
    with _$StateNodeConfig<TValue, TEvent, TContext> {
  factory StateNodeConfig.atomic({
    required List<TransitionImmediate<TValue, TEvent, TContext>> immediate,
    required List<TransitionEvent<TValue, TEvent, TContext>> transition,
    required List<EffectEnter<TValue, TEvent, TContext>> effect,
    ReduceOnEnter<TValue, TEvent, TContext>? reduceOnEnter,
    ReduceOnExit<TValue, TEvent, TContext>? reduceOnExit,
    ManagedMachineConfig<TValue, TEvent, TContext>? managed,
  }) = StateNodeConfigAtomic<TValue, TEvent, TContext>;
}

@freezed
class ManagedMachineConfig<TValue, TEvent, TContext>
    with _$ManagedMachineConfig<TValue, TEvent, TContext> {
  factory ManagedMachineConfig({
    required List<
            ManagedMachineItemConfig<TValue, TEvent, TContext, dynamic,
                dynamic>>
        items,
    void Function(List<MachineState<dynamic, Object?, dynamic>> states,
            void Function(TEvent event) send)?
        anyChange,
  }) = _ManagedMachineConfig<TValue, TEvent, TContext>;
}

@freezed
class ManagedMachineItemConfig<TValue, TEvent, TContext, ResValue, ResContext>
    with
        _$ManagedMachineItemConfig<TValue, TEvent, TContext, ResValue,
            ResContext> {
  factory ManagedMachineItemConfig({
    required ManagedMachine<ResValue, Object?, ResContext> machine,
    ResContext Function(MachineState<TValue, TEvent, TContext> state)? context,
    ResValue Function(MachineState<TValue, TEvent, TContext> state)? value,
  }) =
      _ManagedMachineItemConfig<TValue, TEvent, TContext, ResValue, ResContext>;
}
